from vonage_jwt.jwt import JwtClient, VonageJwtError

import os
from pytest import raises
from jwt import decode
from jwt.exceptions import ImmatureSignatureError
from time import time

# Ensure the client isn't being configured with real values
os.environ.clear()


def read_file(path):
    with open(os.path.join(os.path.dirname(__file__), path)) as input_file:
        return input_file.read()


application_id = 'asdf1234'
private_key_string = read_file('data/private_key.txt')
private_key_file_path = './tests/data/private_key.txt'
jwt_client = JwtClient(application_id, private_key_file_path)

public_key = read_file('data/public_key.txt')


def test_create_jwt_client_key_string():
    jwt_client = JwtClient(application_id, private_key_string)
    assert jwt_client._application_id == application_id
    assert jwt_client._private_key == private_key_string


def test_create_jwt_client_key_file():
    jwt_client = JwtClient(application_id, private_key_file_path)
    assert jwt_client._application_id == application_id
    print(jwt_client._private_key)
    print(private_key_string)
    assert jwt_client._private_key == bytes(private_key_string, 'utf-8')


def test_create_jwt_client_error_incomplete():
    with raises(VonageJwtError) as err:
        JwtClient(application_id, None)
    assert str(err.value) == 'Both of "application_id" and "private_key" are required.'


def test_create_jwt_client_error_invalid_key():
    with raises(VonageJwtError) as err:
        JwtClient(application_id, 'invalid-private-key-string')
    assert (
        str(err.value)
        == 'If passing the private key directly as a string, it must be formatted correctly with newlines.'
    )


def test_generate_application_jwt_basic():
    jwt = jwt_client.generate_application_jwt()
    decoded_jwt = decode(jwt, key=public_key, algorithms='RS256')
    assert decoded_jwt['application_id'] == 'asdf1234'
    assert decoded_jwt['exp'] - decoded_jwt['iat'] == 15 * 60


def test_generate_application_jwt_custom_claims():
    now = int(time())
    print(now)
    claims = {'jti': 'qwerasdfzxcv1234', 'nbf': now + 100}
    jwt = jwt_client.generate_application_jwt(claims)
    with raises(ImmatureSignatureError) as err:
        decode(jwt, key=public_key, algorithms='RS256')
    assert str(err.value) == 'The token is not yet valid (nbf)'



import re
from time import time
from jwt import encode
from uuid import uuid4
from typing import Union


class JwtClient:
    """Object used to pass in an application ID and private key to the JWT generator."""

    def __init__(self, application_id: str, private_key: str):
        self._application_id = application_id

        try:
            self._set_private_key(private_key)
        except Exception as err:
            raise VonageJwtError(err)

        if self._application_id is None or self._private_key is None:
            raise VonageJwtError('Both of "application_id" and "private_key" are required.')

    def generate_application_jwt(self, jwt_options: dict = {}):
        """Generates a JWT for the specified Vonage application.
        If values for application_id and private_key are set on the JWTClient object,
        this method will use them. Otherwise, they can be specified directly.
        """

        iat = int(time())

        payload = jwt_options
        payload["application_id"] = self._application_id
        payload.setdefault("iat", iat)
        payload.setdefault("jti", str(uuid4()))
        payload.setdefault("exp", iat + (15 * 60))

        headers = {'alg': 'RS256', 'typ': 'JWT'}

        token = encode(payload, self._private_key, algorithm="RS256", headers=headers)
        return bytes(token, 'utf-8')

    def _set_private_key(self, key: Union[str, bytes]):
        if isinstance(key, (str, bytes)) and re.search("[.][a-zA-Z0-9_]+$", key):
            with open(key, "rb") as key_file:
                self._private_key = key_file.read()
        elif isinstance(key, str) and '-----BEGIN PRIVATE KEY-----' not in key:
            raise VonageJwtError(
                "If passing the private key directly as a string, it must be formatted correctly with newlines."
            )
        else:
            self._private_key = key


class VonageJwtError(Exception):
    """An error relating to the Vonage JWT Generator."""
